%% Load the data
load ../data/ocr_data.mat
goodFeatures=process_data(trainset);
% EXPLANATION OF THE DATA:
%
%  trainset.letter = the actual letters
%  trainset.prev_letter = the previous letter (-1 means start)
%  trainset.pixels = pixel values
%  trainset.wordidx{i} = row indices of the i'th word in the other fields
%
% You can plot the pixels of the i'th character in image form as follows:
% >> imagesc(reshape(trainset.pixels(i,:), 8, 8)'); axis image; colormap gray;

%% Train CRF and LOGREG, plot the model
crf                 =   crf_learn(goodFeatures,reg,trainset.letter,trainset.wordidx,trainset.pixels);
crf.paramsLogReg    =   logreg_learn(goodFeatures,reg,trainset.letter,trainset.pixels);

%%
% Plot transition matrix
figure('Name', 'Transition Probabilities');
letters = arrayfun(@(x){char(x)}, 97:97+25);
imagesc(crf.paramsEdge); colormap gray;
set(gca,'XTickLabel', letters);
set(gca,'XTick', 1:26);
set(gca,'YTickLabel', letters);
set(gca,'YTick', 1:26);
xlabel('Next letter');
ylabel('Previous Letter');
title('Transition Potential');
print -djpeg -r72 plot_1.1.jpg;

% Plot observation probabilities
figure('Name', 'Observation Potential');
%paramsNode=zeros(64,26);
%paramsNode(goodFeatures.nodes)=crf.paramsNode;
p = reshape(crf.paramsNode(:,1),[8, 8])'; 
imagesc(p); colormap gray; 
set(gca,'XTick',[]);
set(gca,'YTick',[]);
title('Node Potential for letter ''a''');
print -djpeg -r72 plot_1.2.jpg;

%% Test CRF and LogReg
t = CTimeleft(numel(testset.wordidx));
errs = zeros(size(testset.wordidx));
errs_logreg = zeros(size(testset.wordidx));
for i = 1:numel(testset.wordidx)
    t.timeleft();
    
    % Test CRF model
    idx = testset.wordidx{i};

    % Compute marginal probabilities of letters and take predictions
    px  = crf.infer(testset.pixels(idx, :)');

    % Take maximum probability states at each point
    [~,yhat_crf] = max(px);

    % Compute probability according to LogReg 
    px_logreg = (testset.pixels(idx,:)*crf.paramsLogReg)';% YOUR CODE GOES HERE, use crf.paramsLogReg
    
    [~,yhat_logreg] = max(px_logreg);
    
    % Compute errors on this word
    errs(i) = sum(yhat_crf ~= testset.letter(idx)');
    errs_logreg(i) = sum(yhat_logreg ~= testset.letter(idx)');
end

% Compute accuracies
n = numel(testset.letter);
fprintf('CRF Letter Accuracy: %g%%\n', (1-sum(errs)/n)*100);
fprintf('CRF Word Accuracy: %g%%\n', (1-mean(errs>0))*100);
fprintf('LogReg Letter Accuracy: %g%%\n', (1-sum(errs_logreg)/n)*100);
fprintf('LogReg Word Accuracy: %g%%\n', (1-mean(errs_logreg>0))*100);


