function params = logreg_learn(goodFeatures,reg,letters,pixels)
% Evaluates the objective (log-likelihood of the data)
%
% Usage:
%
%    params = logreg_learn(goodFeatures,reg,letters,pixels)
%
% Fine MLE parameters for logistic regression.
% goodFeatures is a 64 x 26 logical array indexing good features.
% If M is the number of characters in the dataset, then LETTER is a M x 1 array of letter labels,
% PIXELS is a M x 64 binary matrix of pixel activations for each letter

% The output is the MLE paramters for logistic regression - 64 x 26 array
%

goodFeatures    =   goodFeatures.nodes;
indParams       =   [0,cumsum(sum(goodFeatures))];
np              =   sum(sum(goodFeatures)); %No params for last label%

%Optimize likelihood to get parameters%
paramsopt          =    minFunc(@EvalLikl,zeros(np,1));

%Reshape parameters
params                   =   zeros(64,26);
params(goodFeatures)     =   paramsopt(1:np);

    function [logLikl,gradient]=EvalLikl(params)
        logLikl         =   0;
        gradient        =   zeros(size(params));
        
        %Loop over each word in the dataset
        for it=1:size(letters)
            xs           =   pixels(it,:)';
            ys           =   letters(it);
            
            [lik,grad]   =   inferlikelihood(params,xs,ys);
            
            logLikl     =   logLikl+lik;
            gradient    =   gradient+grad;
        end
        
        %Scale Objective and Gradient by number of samples
        logLikl =   logLikl/length(letters);
        gradient=   gradient/length(letters);
        
        %Negate likelihood/gradient for minimization
        
        logLikl     =   -logLikl;
        gradient    =   -gradient;
        
        %Add Tikhonov regularization
        
        logLikl     =   logLikl+params'*params*reg/2;
        gradient    =   gradient+reg*params;
    end

% YOU need to implement the function inferlikelihood:
% Evaluate likelihood of character label given image pix

%To be implemented by the student%
    function [l,g]=inferlikelihood(params,pix,label)
        l=0;
        g=zeros(size(params));
    end
end
